@@ -1,11 +1,15 @@
+-file("compiler/diameter_dict_parser.yrl", 0).
 -module(diameter_dict_parser).
+-file("gen/diameter_dict_parser.erl", 3).
 -export([parse/1, parse_and_scan/1, format_error/1]).
+-file("compiler/diameter_dict_parser.yrl", 327).
+-moduledoc false.
 
 -file("/build/erlang-nox/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,10 +31,16 @@
 
 -type yecc_ret() :: {'error', _} | {'ok', _}.
 
+-ifdef (YECC_PARSE_DOC).
+-doc ?YECC_PARSE_DOC.
+-endif.
 -spec parse(Tokens :: list()) -> yecc_ret().
 parse(Tokens) ->
     yeccpars0(Tokens, {no_func, no_location}, 0, [], []).
 
+-ifdef (YECC_PARSE_AND_SCAN_DOC).
+-doc ?YECC_PARSE_AND_SCAN_DOC.
+-endif.
 -spec parse_and_scan({function() | {atom(), atom()}, [_]}
                      | {atom(), atom(), [_]}) -> yecc_ret().
 parse_and_scan({F, A}) ->
@@ -39,6 +49,9 @@ parse_and_scan({M, F, A}) ->
     Arity = length(A),
     yeccpars0([], {{fun M:F/Arity, A}, no_location}, 0, [], []).
 
+-ifdef (YECC_FORMAT_ERROR_DOC).
+-doc ?YECC_FORMAT_ERROR_DOC.
+-endif.
 -spec format_error(any()) -> [char() | list()].
 format_error(Message) ->
     case io_lib:deep_char_list(Message) of
@@ -180,7 +193,7 @@ yecctoken2string1(Other) ->
 
 
 
--file("gen/diameter_dict_parser.erl", 183).
+-file("gen/diameter_dict_parser.erl", 196).
 
 -dialyzer({nowarn_function, yeccpars2/7}).
 -compile({nowarn_unused_function,  yeccpars2/7}).
@@ -453,31 +466,31 @@ yeccpars2(Other, _, _, _, _, _, _) ->
 
 -dialyzer({nowarn_function, yeccpars2_0/7}).
 -compile({nowarn_unused_function,  yeccpars2_0/7}).
-yeccpars2_0(S, avp_types, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'avp_types', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 4, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, avp_vendor_id, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'avp_vendor_id', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, codecs, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'codecs', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, custom_types, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'custom_types', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 7, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, define, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'define', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 8, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, enum, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'enum', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 9, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, grouped, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'grouped', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 10, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, id, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'id', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 11, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, inherits, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'inherits', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 12, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, messages, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'messages', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 13, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, name, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'name', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 14, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, prefix, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'prefix', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 15, Ss, Stack, T, Ts, Tzr);
-yeccpars2_0(S, vendor, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_0(S, 'vendor', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 16, Ss, Stack, T, Ts, Tzr);
 yeccpars2_0(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_0_(Stack),
@@ -491,31 +504,31 @@ yeccpars2_1(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_2/7}).
 -compile({nowarn_unused_function,  yeccpars2_2/7}).
-yeccpars2_2(S, avp_types, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'avp_types', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 4, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, avp_vendor_id, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'avp_vendor_id', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 5, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, codecs, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'codecs', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, custom_types, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'custom_types', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 7, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, define, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'define', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 8, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, enum, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'enum', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 9, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, grouped, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'grouped', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 10, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, id, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'id', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 11, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, inherits, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'inherits', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 12, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, messages, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'messages', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 13, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, name, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'name', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 14, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, prefix, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'prefix', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 15, Ss, Stack, T, Ts, Tzr);
-yeccpars2_2(S, vendor, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_2(S, 'vendor', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 16, Ss, Stack, T, Ts, Tzr);
 yeccpars2_2(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_2_(Stack),
@@ -530,7 +543,7 @@ yeccpars2_3(_, _, _, _, T, _, _) ->
 
 -dialyzer({nowarn_function, yeccpars2_4/7}).
 -compile({nowarn_unused_function,  yeccpars2_4/7}).
-yeccpars2_4(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_4(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_4(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_4_(Stack),
@@ -538,14 +551,14 @@ yeccpars2_4(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_5/7}).
 -compile({nowarn_unused_function,  yeccpars2_5/7}).
-yeccpars2_5(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_5(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 119, Ss, Stack, T, Ts, Tzr);
 yeccpars2_5(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
 -dialyzer({nowarn_function, yeccpars2_6/7}).
 -compile({nowarn_unused_function,  yeccpars2_6/7}).
-yeccpars2_6(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_6(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_6(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -558,7 +571,7 @@ yeccpars2_6(_, _, _, _, T, _, _) ->
 
 -dialyzer({nowarn_function, yeccpars2_10/7}).
 -compile({nowarn_unused_function,  yeccpars2_10/7}).
-yeccpars2_10(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_10(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_10_(Stack),
@@ -566,7 +579,7 @@ yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_11/7}).
 -compile({nowarn_unused_function,  yeccpars2_11/7}).
-yeccpars2_11(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_11(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 88, Ss, Stack, T, Ts, Tzr);
 yeccpars2_11(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -577,7 +590,7 @@ yeccpars2_11(_, _, _, _, T, _, _) ->
 -compile({nowarn_unused_function,  yeccpars2_13/7}).
 yeccpars2_13(S, 'answer-message', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 26, Ss, Stack, T, Ts, Tzr);
-yeccpars2_13(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_13(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_13(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_13_(Stack),
@@ -589,7 +602,7 @@ yeccpars2_13(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_16/7}).
 -compile({nowarn_unused_function,  yeccpars2_16/7}).
-yeccpars2_16(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_16(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 17, Ss, Stack, T, Ts, Tzr);
 yeccpars2_16(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -647,7 +660,7 @@ yeccpars2_24(_, _, _, _, T, _, _) ->
 -compile({nowarn_unused_function,  yeccpars2_25/7}).
 yeccpars2_25(S, 'answer-message', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 26, Ss, Stack, T, Ts, Tzr);
-yeccpars2_25(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_25(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_25(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_25_(Stack),
@@ -705,7 +718,7 @@ yeccpars2_32(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_33/7}).
 -compile({nowarn_unused_function,  yeccpars2_33/7}).
-yeccpars2_33(S, code, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_33(S, 'code', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 34, Ss, Stack, T, Ts, Tzr);
 yeccpars2_33(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -760,7 +773,7 @@ yeccpars2_40(S, '<', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 46, Ss, Stack, T, Ts, Tzr);
 yeccpars2_40(S, '[', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 47, Ss, Stack, T, Ts, Tzr);
-yeccpars2_40(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_40(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 48, Ss, Stack, T, Ts, Tzr);
 yeccpars2_40(S, '{', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 49, Ss, Stack, T, Ts, Tzr);
@@ -800,7 +813,7 @@ yeccpars2_44(S, '<', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 46, Ss, Stack, T, Ts, Tzr);
 yeccpars2_44(S, '[', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 47, Ss, Stack, T, Ts, Tzr);
-yeccpars2_44(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_44(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 48, Ss, Stack, T, Ts, Tzr);
 yeccpars2_44(S, '{', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 49, Ss, Stack, T, Ts, Tzr);
@@ -810,7 +823,7 @@ yeccpars2_44(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_45/7}).
 -compile({nowarn_unused_function,  yeccpars2_45/7}).
-yeccpars2_45(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_45(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 61, Ss, Stack, T, Ts, Tzr);
 yeccpars2_45(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_45_(Stack),
@@ -822,7 +835,7 @@ yeccpars2_45(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 -compile({nowarn_unused_function,  yeccpars2_47/7}).
 yeccpars2_47(S, 'AVP', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 53, Ss, Stack, T, Ts, Tzr);
-yeccpars2_47(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_47(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_47(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -870,7 +883,7 @@ yeccpars2_54(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_55/7}).
 -compile({nowarn_unused_function,  yeccpars2_55/7}).
-yeccpars2_55(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_55(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 56, Ss, Stack, T, Ts, Tzr);
 yeccpars2_55(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  [_|Nss] = Ss,
@@ -955,7 +968,7 @@ yeccpars2_66(S, '<', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 46, Ss, Stack, T, Ts, Tzr);
 yeccpars2_66(S, '[', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 47, Ss, Stack, T, Ts, Tzr);
-yeccpars2_66(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_66(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 48, Ss, Stack, T, Ts, Tzr);
 yeccpars2_66(S, '{', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 49, Ss, Stack, T, Ts, Tzr);
@@ -974,7 +987,7 @@ yeccpars2_68(_, _, _, _, T, _, _) ->
 
 -dialyzer({nowarn_function, yeccpars2_69/7}).
 -compile({nowarn_unused_function,  yeccpars2_69/7}).
-yeccpars2_69(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_69(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 71, Ss, Stack, T, Ts, Tzr);
 yeccpars2_69(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -995,7 +1008,7 @@ yeccpars2_71(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_72/7}).
 -compile({nowarn_unused_function,  yeccpars2_72/7}).
-yeccpars2_72(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_72(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 79, Ss, Stack, T, Ts, Tzr);
 yeccpars2_72(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_72_(Stack),
@@ -1066,7 +1079,7 @@ yeccpars2_81(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_82/7}).
 -compile({nowarn_unused_function,  yeccpars2_82/7}).
-yeccpars2_82(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_82(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_82(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_82_(Stack),
@@ -1086,7 +1099,7 @@ yeccpars2_84(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_85/7}).
 -compile({nowarn_unused_function,  yeccpars2_85/7}).
-yeccpars2_85(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_85(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_85(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_85_(Stack),
@@ -1129,7 +1142,7 @@ yeccpars2_90(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_91/7}).
 -compile({nowarn_unused_function,  yeccpars2_91/7}).
-yeccpars2_91(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_91(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_91(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_91_(Stack),
@@ -1157,7 +1170,7 @@ yeccpars2_94(S, '<', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 46, Ss, Stack, T, Ts, Tzr);
 yeccpars2_94(S, '[', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 47, Ss, Stack, T, Ts, Tzr);
-yeccpars2_94(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_94(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 48, Ss, Stack, T, Ts, Tzr);
 yeccpars2_94(S, '{', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 49, Ss, Stack, T, Ts, Tzr);
@@ -1203,14 +1216,14 @@ yeccpars2_99(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_100/7}).
 -compile({nowarn_unused_function,  yeccpars2_100/7}).
-yeccpars2_100(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_100(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 102, Ss, Stack, T, Ts, Tzr);
 yeccpars2_100(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
 -dialyzer({nowarn_function, yeccpars2_101/7}).
 -compile({nowarn_unused_function,  yeccpars2_101/7}).
-yeccpars2_101(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_101(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 104, Ss, Stack, T, Ts, Tzr);
 yeccpars2_101(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_101_(Stack),
@@ -1251,7 +1264,7 @@ yeccpars2_106(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_107/7}).
 -compile({nowarn_unused_function,  yeccpars2_107/7}).
-yeccpars2_107(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_107(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_107(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_107_(Stack),
@@ -1259,7 +1272,7 @@ yeccpars2_107(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_108/7}).
 -compile({nowarn_unused_function,  yeccpars2_108/7}).
-yeccpars2_108(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_108(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 112, Ss, Stack, T, Ts, Tzr);
 yeccpars2_108(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -1273,7 +1286,7 @@ yeccpars2_109(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_110/7}).
 -compile({nowarn_unused_function,  yeccpars2_110/7}).
-yeccpars2_110(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_110(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_110(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_110_(Stack),
@@ -1295,7 +1308,7 @@ yeccpars2_112(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_113/7}).
 -compile({nowarn_unused_function,  yeccpars2_113/7}).
-yeccpars2_113(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_113(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_113(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_113_(Stack),
@@ -1310,7 +1323,7 @@ yeccpars2_114(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_115/7}).
 -compile({nowarn_unused_function,  yeccpars2_115/7}).
-yeccpars2_115(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_115(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_115(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_115_(Stack),
@@ -1325,7 +1338,7 @@ yeccpars2_116(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_117/7}).
 -compile({nowarn_unused_function,  yeccpars2_117/7}).
-yeccpars2_117(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_117(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_117(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_117_(Stack),
@@ -1340,7 +1353,7 @@ yeccpars2_118(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_119/7}).
 -compile({nowarn_unused_function,  yeccpars2_119/7}).
-yeccpars2_119(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_119(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_119(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_119_(Stack),
@@ -1355,7 +1368,7 @@ yeccpars2_120(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_121/7}).
 -compile({nowarn_unused_function,  yeccpars2_121/7}).
-yeccpars2_121(S, number, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_121(S, 'number', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 125, Ss, Stack, T, Ts, Tzr);
 yeccpars2_121(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -1369,7 +1382,7 @@ yeccpars2_122(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 -dialyzer({nowarn_function, yeccpars2_123/7}).
 -compile({nowarn_unused_function,  yeccpars2_123/7}).
-yeccpars2_123(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_123(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_123(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_123_(Stack),
@@ -1394,7 +1407,7 @@ yeccpars2_126(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 -compile({nowarn_unused_function,  yeccpars2_127/7}).
 yeccpars2_127(S, '-', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 130, Ss, Stack, T, Ts, Tzr);
-yeccpars2_127(S, word, Ss, Stack, T, Ts, Tzr) ->
+yeccpars2_127(S, 'word', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 19, Ss, Stack, T, Ts, Tzr);
 yeccpars2_127(_, _, _, _, T, _, _) ->
  yeccerror(T).
@@ -2533,3 +2546,4 @@ yeccpars2_131_(__Stack0) ->
   end | __Stack].
 
 
+-file("compiler/diameter_dict_parser.yrl", 329).
