@@ -1,11 +1,15 @@
+-file("xmerl_b64Bin.yrl", 0).
 -module(xmerl_b64Bin).
+-file("xmerl_b64Bin.erl", 3).
 -export([parse/1, parse_and_scan/1, format_error/1]).
+-file("xmerl_b64Bin.yrl", 71).
+-moduledoc false.
 
 -file("/build/erlang-nox/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,10 +31,16 @@
 
 -type yecc_ret() :: {'error', _} | {'ok', _}.
 
+-ifdef (YECC_PARSE_DOC).
+-doc ?YECC_PARSE_DOC.
+-endif.
 -spec parse(Tokens :: list()) -> yecc_ret().
 parse(Tokens) ->
     yeccpars0(Tokens, {no_func, no_location}, 0, [], []).
 
+-ifdef (YECC_PARSE_AND_SCAN_DOC).
+-doc ?YECC_PARSE_AND_SCAN_DOC.
+-endif.
 -spec parse_and_scan({function() | {atom(), atom()}, [_]}
                      | {atom(), atom(), [_]}) -> yecc_ret().
 parse_and_scan({F, A}) ->
@@ -39,6 +49,9 @@ parse_and_scan({M, F, A}) ->
     Arity = length(A),
     yeccpars0([], {{fun M:F/Arity, A}, no_location}, 0, [], []).
 
+-ifdef (YECC_FORMAT_ERROR_DOC).
+-doc ?YECC_FORMAT_ERROR_DOC.
+-endif.
 -spec format_error(any()) -> [char() | list()].
 format_error(Message) ->
     case io_lib:deep_char_list(Message) of
@@ -180,7 +193,7 @@ yecctoken2string1(Other) ->
 
 
 
--file("xmerl_b64Bin.erl", 183).
+-file("xmerl_b64Bin.erl", 196).
 
 -dialyzer({nowarn_function, yeccpars2/7}).
 -compile({nowarn_unused_function,  yeccpars2/7}).
@@ -543,3 +556,4 @@ yeccpars2_17_(__Stack0) ->
   end | __Stack].
 
 
+-file("xmerl_b64Bin.yrl", 73).
